==> superd-0.2.tar.gz: superd-0.2/cmd/superctl/main.go <==
// Copyright 2022 Clayton Craft <clayton@craftyguy.net>
// SPDX-License-Identifier: GPL-3.0-or-later

package main

import (
	"encoding/json"
	"fmt"
	"io/ioutil"
	"log" // Note: *not* using internal/log here because we don't want its formatting
	"net"
	"os"
	"sort"
	"strconv"
	"strings"
	"time"

	"github.com/integrii/flaggy"
	"github.com/olekukonko/tablewriter"
	"sr.ht/~craftyguy/superd/internal/color"
	"sr.ht/~craftyguy/superd/pkg/api"
)

// set at build time
var Version string

var logger *log.Logger

func fatalError(msg string) {
	logger.Fatal("error received from superd:\n" + msg)
}

func main() {
	// disable timestamps, etc in logger output
	logger = log.Default()
	logger.SetFlags(0)

	// this depends on XDG_RUNTIME_DIR, so make sure it is set and fail fast if
	// not
	if v, ok := os.LookupEnv("XDG_RUNTIME_DIR"); v == "" || !ok {
		logger.Fatal("$XDG_RUNTIME_DIR is not set in the environment, this is required.")
	}

	flaggy.SetName("superctl")
	flaggy.SetDescription("Control app for the superd supervisor")
	flaggy.SetVersion(Version)

	// Subcommands
	startCmd := flaggy.NewSubcommand("start")
	startCmd.Description = "Start the given service"
	flaggy.AttachSubcommand(startCmd, 1)

	stopCmd := flaggy.NewSubcommand("stop")
	stopCmd.Description = "Stop the given service"
	flaggy.AttachSubcommand(stopCmd, 1)

	restartCmd := flaggy.NewSubcommand("restart")
	restartCmd.Description = "Restart the given service"
	flaggy.AttachSubcommand(restartCmd, 1)

	enableCmd := flaggy.NewSubcommand("enable")
	enableCmd.Description = "Enable the given service"
	flaggy.AttachSubcommand(enableCmd, 1)

	disableCmd := flaggy.NewSubcommand("disable")
	disableCmd.Description = "Disable the given service"
	flaggy.AttachSubcommand(disableCmd, 1)

	statusCmd := flaggy.NewSubcommand("status")
	statusCmd.Description = "Get status for the give service(s), or all services if not specified"
	flaggy.AttachSubcommand(statusCmd, 1)

	setEnvCmd := flaggy.NewSubcommand("set-env")
	setEnvCmd.Description = "Set one or more environment variables for running services, either to the value specified or the value from the current environment if it is set"
	flaggy.AttachSubcommand(setEnvCmd, 1)

	reloadCmd := flaggy.NewSubcommand("reload")
	reloadCmd.Description = "Reload service config from disk, adding any new config found"
	flaggy.AttachSubcommand(reloadCmd, 1)

	// Subcommand flags
	var now bool
	enableCmd.Bool(&now, "n", "now", "Start service after enabling it")
	disableCmd.Bool(&now, "n", "now", "Stop service after disabling it")

	// Subcommand positional args
	var service string
	startCmd.AddPositionalValue(&service, "service", 1, true, "Service to start")
	stopCmd.AddPositionalValue(&service, "service", 1, true, "Service to stop")
	restartCmd.AddPositionalValue(&service, "service", 1, true, "Service to restart")
	enableCmd.AddPositionalValue(&service, "service", 1, true, "Service to enable")
	disableCmd.AddPositionalValue(&service, "service", 1, true, "Service to disable")
	statusCmd.AddPositionalValue(&service, "service", 1, false, "Service to get status for, or all if not specified")

	flaggy.Parse()

	socketPath := os.ExpandEnv("$XDG_RUNTIME_DIR/superd.sock")

	if startCmd.Used {

		startService(service, socketPath)

	} else if stopCmd.Used {

		stopService(service, socketPath)

	} else if restartCmd.Used {
		var resp api.NilResponse

		resBuf, err := sendCmd(socketPath, api.CmdRestart, []string{service})
		if err != nil {
			logger.Fatal(err)
		}

		if err := json.Unmarshal(resBuf, &resp); err != nil {
			logger.Fatal(err)
		}

		if resp.Error {
			fatalError(resp.ErrorMsg)
		}

	} else if statusCmd.Used {
		var resp api.ServiceDetailsResponse

		// if service is an empty string then the user wanted status for all
		// services, so make sure to send an empty list (and not a list with an
		// empty string...) to superd
		var arg []string
		if service != "" {
			arg = append(arg, service)
		}

		resBuf, err := sendCmd(socketPath, api.CmdStatus, arg)
		if err != nil {
			logger.Fatal(err)
		}

		if err := json.Unmarshal(resBuf, &resp); err != nil {

			logger.Fatal(err)
		}

		if resp.Error {
			fatalError(resp.ErrorMsg)
		}

		if len(resp.Result) == 0 {
			logger.Fatal("no statuses were returned by superd")
		} else if len(resp.Result) == 1 {
			// print detailed output for 1 service
			if err := printStatusSingle(service, resp.Result[service]); err != nil {
				logger.Fatal(err)
			}
		} else {
			// print less detailed status for many services
			if err := printStatusMulti(resp.Result); err != nil {
				logger.Fatal(err)
			}
		}

	} else if enableCmd.Used {
		var resp api.StringResponse

		args := []string{service}
		resBuf, err := sendCmd(socketPath, api.CmdEnable, args)
		if err != nil {
			logger.Fatal(err)
		}

		if err := json.Unmarshal(resBuf, &resp); err != nil {
			logger.Fatal(err)
		}

		if resp.Error {
			fatalError(resp.ErrorMsg)
		}

		logger.Println(resp.Result)

		if now {
			startService(service, socketPath)
		}

	} else if disableCmd.Used {
		var resp api.StringResponse

		args := []string{service}
		resBuf, err := sendCmd(socketPath, api.CmdDisable, args)
		if err != nil {
			logger.Fatal(err)
		}

		if err := json.Unmarshal(resBuf, &resp); err != nil {
			logger.Fatal(err)
		}

		if resp.Error {
			fatalError(resp.ErrorMsg)
		}

		logger.Println(resp.Result)

		if now {
			stopService(service, socketPath)
		}

	} else if setEnvCmd.Used {
		var resp api.NilResponse

		args := []string{}

		// TODO: make a patch to flaggy to fix this nonsense:
		// https://github.com/integrii/flaggy/issues/62
		if len(flaggy.TrailingArguments) == 0 {
			flaggy.ShowHelpAndExit("This command requires specifying -- before listing variables, e.g: set-env -- FOO=1 BAR=bazz DISPLAY APPLES=nope")
		}

		// set values for any env vars where the value was not passed on the
		// cmd line
		for _, v := range flaggy.TrailingArguments {
			if !strings.ContainsRune(v, '=') {
				// if it's unset in the current env, just allow it to be set to
				// an empty string. i.e. the user may want the var to still be
				// present in the environment.
				v = fmt.Sprintf("%s=%s", v, os.Getenv(v))
			}
			args = append(args, v)
		}

		resBuf, err := sendCmd(socketPath, api.CmdSetEnv, args)
		if err != nil {
			logger.Fatal(err)
		}

		if err := json.Unmarshal(resBuf, &resp); err != nil {
			logger.Fatal(err)
		}

		if resp.Error {
			fatalError(resp.ErrorMsg)
		}

	} else if reloadCmd.Used {
		var resp api.NilResponse

		resBuf, err := sendCmd(socketPath, api.CmdReload, []string{})
		if err != nil {
			logger.Fatal(err)
		}

		if err := json.Unmarshal(resBuf, &resp); err != nil {
			logger.Fatal(err)
		}

		if resp.Error {
			fatalError(resp.ErrorMsg)
		}
	}
}

func startService(service string, socketPath string) {

	var resp api.NilResponse

	resBuf, err := sendCmd(socketPath, api.CmdStart, []string{service})
	if err != nil {
		logger.Fatal(err)
	}

	if err := json.Unmarshal(resBuf, &resp); err != nil {
		logger.Fatal(err)
	}

	if resp.Error {
		fatalError(resp.ErrorMsg)
	}
}

func stopService(service string, socketPath string) {

	resBuf, err := sendCmd(socketPath, api.CmdStop, []string{service})
	if err != nil {
		logger.Fatal(err)
	}

	var resp api.NilResponse
	if err := json.Unmarshal(resBuf, &resp); err != nil {
		logger.Fatal(err)
	}

	if resp.Error {
		fatalError(resp.ErrorMsg)
	}
}

func sendCmd(socketPath string, cmd api.ReqCommand, args []string) (resp []byte, err error) {
	sock, err := net.Dial("unix", socketPath)
	if err != nil {
		err = fmt.Errorf("unable to open socket, is superd running?\n%w", err)
		return
	}
	defer sock.Close()

	req := api.Request{
		Cmd:    cmd,
		Params: args,
	}

	reqBuf, err := req.Json()
	if err != nil {
		return
	}

	sock.SetDeadline(time.Now().Add(2 * time.Second))

	if _, err = sock.Write(reqBuf); err != nil {
		return
	}

	sock.SetDeadline(time.Now().Add(2 * time.Second))

	resp, err = ioutil.ReadAll(sock)

	return
}

func printStatusMulti(services map[string]api.ServiceDetails) error {
	// sort services in response
	var keys []string
	for k := range services {
		keys = append(keys, k)
	}
	sort.Strings(keys)

	rows := [][]string{}

	for _, k := range keys {
		var (
			status string
			pid    string = "-"
			auto   string = "-"
		)

		switch services[k].Status {
		case api.ServiceFailed:
			status = color.Red(services[k].Status)
		case api.ServiceStarted:
			status = color.Green(services[k].Status)
			pid = strconv.Itoa((services[k].Pid))
		case api.ServiceStopped:
			status = color.Yellow(services[k].Status)
		default:
			status = string(services[k].Status)
		}

		if services[k].Autostart {
			auto = "✓"
		}

		rows = append(rows, []string{k, status, pid, auto})
	}

	table := newTable()
	table.SetHeader([]string{"", "Status", "PID", "Autostart"})
	table.AppendBulk(rows)
	table.Render()
	return nil
}

func printStatusSingle(name string, details api.ServiceDetails) error {

	var (
		status string
		auto   string = "-"
		pid    string = "-"
	)

	switch details.Status {
	case api.ServiceFailed:
		status = color.Red(details.Status)
	case api.ServiceStarted:
		status = color.Green(details.Status)
		pid = strconv.Itoa(details.Pid)
	case api.ServiceStopped:
		status = color.Yellow(details.Status)
	default:
		status = string(details.Status)
	}

	if details.Autostart {
		auto = "✓"
	}

	table := newTable()
	table.SetHeader([]string{name, "", ""})
	table.AppendBulk([][]string{
		{"", "Status:", status},
		{"", "Type:", details.Type},
		{"", "PID:", pid},
		{"", "Autostart:", auto},
		{"", "Loaded:", details.ConfigPath},
	})
	table.Render()

	return nil
}

// helper that returns a table configured to display as little
// decoration/border/padding as possible
func newTable() (table *tablewriter.Table) {
	table = tablewriter.NewWriter(os.Stdout)
	table.SetAutoWrapText(true)
	table.SetRowSeparator("")
	table.SetColumnSeparator("")
	table.SetCenterSeparator("")
	table.SetHeaderLine(false)
	table.SetHeaderAlignment(tablewriter.ALIGN_LEFT)
	table.SetAlignment(tablewriter.ALIGN_LEFT)
	table.SetAutoFormatHeaders(false) // disables all caps for header text
	table.SetBorder(false)
	table.SetNoWhiteSpace(true)
	table.SetTablePadding("   ") // separator between columns
	return table
}

==> superd-0.3.2.tar.gz: superd-0.3.2/cmd/superctl/main.go <==
// Copyright 2022 Clayton Craft <clayton@craftyguy.net>
// SPDX-License-Identifier: GPL-3.0-or-later

package main

import (
	"encoding/json"
	"fmt"
	"io/ioutil"
	"log" // Note: *not* using internal/log here because we don't want its formatting
	"net"
	"os"
	"sort"
	"strconv"
	"strings"
	"time"

	"github.com/integrii/flaggy"
	"github.com/olekukonko/tablewriter"
	"sr.ht/~craftyguy/superd/internal/color"
	"sr.ht/~craftyguy/superd/pkg/api"
)

// set at build time
var Version string

var logger *log.Logger

func fatalError(msg string) {
	logger.Fatal("error received from superd:\n" + msg)
}

func main() {
	// disable timestamps, etc in logger output
	logger = log.Default()
	logger.SetFlags(0)

	// this depends on XDG_RUNTIME_DIR, so make sure it is set and fail fast if
	// not
	if v, ok := os.LookupEnv("XDG_RUNTIME_DIR"); v == "" || !ok {
		logger.Fatal("$XDG_RUNTIME_DIR is not set in the environment, this is required.")
	}

	flaggy.SetName("superctl")
	flaggy.SetDescription("Control app for the superd supervisor")
	flaggy.SetVersion(Version)

	// Subcommands
	startCmd := flaggy.NewSubcommand("start")
	startCmd.Description = "Start the given service"
	flaggy.AttachSubcommand(startCmd, 1)

	stopCmd := flaggy.NewSubcommand("stop")
	stopCmd.Description = "Stop the given service"
	flaggy.AttachSubcommand(stopCmd, 1)

	restartCmd := flaggy.NewSubcommand("restart")
	restartCmd.Description = "Restart the given service"
	flaggy.AttachSubcommand(restartCmd, 1)

	enableCmd := flaggy.NewSubcommand("enable")
	enableCmd.Description = "Enable the given service"
	flaggy.AttachSubcommand(enableCmd, 1)

	disableCmd := flaggy.NewSubcommand("disable")
	disableCmd.Description = "Disable the given service"
	flaggy.AttachSubcommand(disableCmd, 1)

	statusCmd := flaggy.NewSubcommand("status")
	statusCmd.Description = "Get status for the give service(s), or all services if not specified"
	flaggy.AttachSubcommand(statusCmd, 1)

	setEnvCmd := flaggy.NewSubcommand("set-env")
	setEnvCmd.Description = "Set one or more environment variables for running services, either to the value specified or the value from the current environment if it is set"
	flaggy.AttachSubcommand(setEnvCmd, 1)

	reloadCmd := flaggy.NewSubcommand("reload")
	reloadCmd.Description = "Reload service config from disk, adding any new config found"
	flaggy.AttachSubcommand(reloadCmd, 1)

	// Subcommand flags
	var now bool
	enableCmd.Bool(&now, "n", "now", "Start service after enabling it")
	disableCmd.Bool(&now, "n", "now", "Stop service after disabling it")

	// Subcommand positional args
	var service string
	startCmd.AddPositionalValue(&service, "service", 1, true, "Service to start")
	stopCmd.AddPositionalValue(&service, "service", 1, true, "Service to stop")
	restartCmd.AddPositionalValue(&service, "service", 1, true, "Service to restart")
	enableCmd.AddPositionalValue(&service, "service", 1, true, "Service to enable")
	disableCmd.AddPositionalValue(&service, "service", 1, true, "Service to disable")
	statusCmd.AddPositionalValue(&service, "service", 1, false, "Service to get status for, or all if not specified")

	flaggy.Parse()

	socketPath := os.ExpandEnv("$XDG_RUNTIME_DIR/superd.sock")

	if startCmd.Used {

		startService(service, socketPath)

	} else if stopCmd.Used {

		stopService(service, socketPath)

	} else if restartCmd.Used {
		var resp api.NilResponse

		resBuf, err := sendCmd(socketPath, api.CmdRestart, []string{service})
		if err != nil {
			logger.Fatal(err)
		}

		if err := json.Unmarshal(resBuf, &resp); err != nil {
			logger.Fatal(err)
		}

		if resp.Error {
			fatalError(resp.ErrorMsg)
		}

	} else if statusCmd.Used {
		var resp api.ServiceDetailsResponse

		// if service is an empty string then the user wanted status for all
		// services, so make sure to send an empty list (and not a list with an
		// empty string...) to superd
		var arg []string
		if service != "" {
			arg = append(arg, service)
		}

		resBuf, err := sendCmd(socketPath, api.CmdStatus, arg)
		if err != nil {
			logger.Fatal(err)
		}

		if err := json.Unmarshal(resBuf, &resp); err != nil {

			logger.Fatal(err)
		}

		if resp.Error {
			fatalError(resp.ErrorMsg)
		}

		if len(resp.Result) == 0 {
			logger.Fatal("no statuses were returned by superd")
		} else if len(resp.Result) == 1 {
			// print detailed output for 1 service
			if err := printStatusSingle(service, resp.Result[service]); err != nil {
				logger.Fatal(err)
			}
		} else {
			// print less detailed status for many services
			if err := printStatusMulti(resp.Result); err != nil {
				logger.Fatal(err)
			}
		}

	} else if enableCmd.Used {
		var resp api.StringResponse

		args := []string{service}
		resBuf, err := sendCmd(socketPath, api.CmdEnable, args)
		if err != nil {
			logger.Fatal(err)
		}

		if err := json.Unmarshal(resBuf, &resp); err != nil {
			logger.Fatal(err)
		}

		if resp.Error {
			fatalError(resp.ErrorMsg)
		}

		logger.Println(resp.Result)

		if now {
			startService(service, socketPath)
		}

	} else if disableCmd.Used {
		var resp api.StringResponse

		args := []string{service}
		resBuf, err := sendCmd(socketPath, api.CmdDisable, args)
		if err != nil {
			logger.Fatal(err)
		}

		if err := json.Unmarshal(resBuf, &resp); err != nil {
			logger.Fatal(err)
		}

		if resp.Error {
			fatalError(resp.ErrorMsg)
		}

		logger.Println(resp.Result)

		if now {
			stopService(service, socketPath)
		}

	} else if setEnvCmd.Used {
		var resp api.NilResponse

		args := []string{}

		// TODO: make a patch to flaggy to fix this nonsense:
		// https://github.com/integrii/flaggy/issues/62
		if len(flaggy.TrailingArguments) == 0 {
			flaggy.ShowHelpAndExit("This command requires specifying -- before listing variables, e.g: set-env -- FOO=1 BAR=bazz DISPLAY APPLES=nope")
		}

		// set values for any env vars where the value was not passed on the
		// cmd line
		for _, v := range flaggy.TrailingArguments {
			if !strings.ContainsRune(v, '=') {
				// if it's unset in the current env, just allow it to be set to
				// an empty string. i.e. the user may want the var to still be
				// present in the environment.
				v = fmt.Sprintf("%s=%s", v, os.Getenv(v))
			}
			args = append(args, v)
		}

		resBuf, err := sendCmd(socketPath, api.CmdSetEnv, args)
		if err != nil {
			logger.Fatal(err)
		}

		if err := json.Unmarshal(resBuf, &resp); err != nil {
			logger.Fatal(err)
		}

		if resp.Error {
			fatalError(resp.ErrorMsg)
		}

	} else if reloadCmd.Used {
		var resp api.NilResponse

		resBuf, err := sendCmd(socketPath, api.CmdReload, []string{})
		if err != nil {
			logger.Fatal(err)
		}

		if err := json.Unmarshal(resBuf, &resp); err != nil {
			logger.Fatal(err)
		}

		if resp.Error {
			fatalError(resp.ErrorMsg)
		}
	}
}

func startService(service string, socketPath string) {

	var resp api.NilResponse

	resBuf, err := sendCmd(socketPath, api.CmdStart, []string{service})
	if err != nil {
		logger.Fatal(err)
	}

	if err := json.Unmarshal(resBuf, &resp); err != nil {
		logger.Fatal(err)
	}

	if resp.Error {
		fatalError(resp.ErrorMsg)
	}
}

func stopService(service string, socketPath string) {

	resBuf, err := sendCmd(socketPath, api.CmdStop, []string{service})
	if err != nil {
		logger.Fatal(err)
	}

	var resp api.NilResponse
	if err := json.Unmarshal(resBuf, &resp); err != nil {
		logger.Fatal(err)
	}

	if resp.Error {
		fatalError(resp.ErrorMsg)
	}
}

func sendCmd(socketPath string, cmd api.ReqCommand, args []string) (resp []byte, err error) {
	sock, err := net.Dial("unix", socketPath)
	if err != nil {
		err = fmt.Errorf("unable to open socket, is superd running?\n%w", err)
		return
	}
	defer sock.Close()

	req := api.Request{
		Cmd:    cmd,
		Params: args,
	}

	reqBuf, err := req.Json()
	if err != nil {
		return
	}

	sock.SetDeadline(time.Now().Add(2 * time.Second))

	if _, err = sock.Write(reqBuf); err != nil {
		return
	}

	sock.SetDeadline(time.Now().Add(2 * time.Second))

	resp, err = ioutil.ReadAll(sock)

	return
}

func colorStatus(status api.ServiceStatus) string {
	switch status {
	case api.ServiceFailed:
		return color.Red(status)
	case api.ServiceStarted:
		return color.Green(status)
	case api.ServiceStopped:
		return color.Yellow(status)
	default:
		return string(status)
	}
}

func printStatusMulti(services map[string]api.ServiceDetails) error {
	// sort services in response
	var keys []string
	for k := range services {
		keys = append(keys, k)
	}
	sort.Strings(keys)

	rows := [][]string{}

	for _, k := range keys {
		var (
			status  string
			pid     string             = "-"
			auto    string             = "-"
			details api.ServiceDetails = services[k]
		)

		status = colorStatus(details.Status)
		if details.Status == api.ServiceStarted {
			if details.Type == "oneshot" {
				// skip info that doesn't make sense when type is oneshot
				break
			}
			pid = strconv.Itoa((details.Pid))
		}

		if details.Autostart {
			auto = "✓"
		}

		rows = append(rows, []string{k, status, pid, auto})
	}

	table := newTable()
	table.SetHeader([]string{"", "Status", "PID", "Autostart"})
	table.AppendBulk(rows)
	table.Render()
	return nil
}

func printStatusSingle(name string, details api.ServiceDetails) error {

	var (
		status string
		auto   string = "-"
		pid    string = "-"
		uptime string = "-"
	)

	status = colorStatus(details.Status)
	if details.Status == api.ServiceStarted {
		// skip info that doesn't make sense when type is oneshot
		if details.Type != "oneshot" {
			pid = strconv.Itoa(details.Pid)
			uptime = time.Duration(float64(details.Uptime) * float64(time.Second)).String()
		}
	}

	if details.Autostart {
		auto = "✓"
	}

	table := newTable()
	table.AppendBulk([][]string{
		{"", "Status:", status},
		{"", "Type:", details.Type},
		{"", "PID:", pid},
		{"", "Uptime:", uptime},
		{"", "Autostart:", auto},
		{"", "Loaded:", details.ConfigPath},
		{"", "Description:", details.Description},
	})
	fmt.Println(name)
	table.Render()

	return nil
}

// helper that returns a table configured to display as little
// decoration/border/padding as possible
func newTable() (table *tablewriter.Table) {
	table = tablewriter.NewWriter(os.Stdout)
	table.SetAutoWrapText(true)
	table.SetRowSeparator("")
	table.SetColumnSeparator("")
	table.SetCenterSeparator("")
	table.SetHeaderLine(false)
	table.SetHeaderAlignment(tablewriter.ALIGN_LEFT)
	table.SetAlignment(tablewriter.ALIGN_LEFT)
	table.SetAutoFormatHeaders(false) // disables all caps for header text
	table.SetBorder(false)
	table.SetNoWhiteSpace(true)
	table.SetTablePadding("   ") // separator between columns
	return table
}
